@@ -108,7 +108,7 @@ mutt_copy_hdr (FILE *in, FILE *out, LOFF_T off_start, LOFF_T off_end, int flags,
 	if ((flags & CH_UPDATE_IRT) &&
 	    ascii_strncasecmp ("In-Reply-To:", buf, 12) == 0)
 	  continue;
-        if (flags & CH_UPDATE_LABEL &&
+        if ((flags & CH_UPDATE_LABEL) &&
             ascii_strncasecmp ("X-Label:", buf, 8) == 0)
           continue;
 
@@ -218,6 +218,9 @@ mutt_copy_hdr (FILE *in, FILE *out, LOFF_T off_start, LOFF_T off_end, int flags,
       if ((flags & CH_UPDATE_IRT) &&
 	  ascii_strncasecmp ("In-Reply-To:", buf, 12) == 0)
 	continue;
+      if ((flags & CH_UPDATE_LABEL) &&
+          ascii_strncasecmp ("X-Label:", buf, 8) == 0)
+        continue;
 
       /* Find x -- the array entry where this header is to be saved */
       if (flags & CH_REORDER)
@@ -348,6 +351,7 @@ int
 mutt_copy_header (FILE *in, HEADER *h, FILE *out, int flags, const char *prefix)
 {
   char buffer[SHORT_STRING];
+  char *temp_hdr = NULL;
 
   if (h->env)
     flags |= (h->env->irt_changed ? CH_UPDATE_IRT : 0)
@@ -418,13 +422,25 @@ mutt_copy_header (FILE *in, HEADER *h, FILE *out, int flags, const char *prefix)
       fprintf (out, "Lines: %d\n", h->lines);
   }
 
-  if (flags & CH_UPDATE_LABEL)
+  if ((flags & CH_UPDATE_LABEL) && h->env->x_label)
   {
     h->xlabel_changed = 0;
-    if (h->env->x_label != NULL)
-      if (fprintf(out, "X-Label: %s\n", h->env->x_label) !=
-		  10 + strlen(h->env->x_label))
-        return -1;
+    temp_hdr = h->env->x_label;
+    /* env->x_label isn't currently stored with direct references
+     * elsewhere.  Context->label_hash strdups the keys.  But to be
+     * safe, encode a copy */
+    if (!(flags & CH_DECODE))
+    {
+      temp_hdr = safe_strdup (temp_hdr);
+      rfc2047_encode_string (&temp_hdr);
+    }
+    if (mutt_write_one_header (out, "X-Label", temp_hdr,
+                               flags & CH_PREFIX ? prefix : 0,
+                               mutt_window_wrap_cols (MuttIndexWindow, Wrap),
+                               flags) == -1)
+      return -1;
+    if (!(flags & CH_DECODE))
+      FREE (&temp_hdr);
   }
 
   if ((flags & CH_NONEWLINE) == 0)
