@@ -917,7 +917,10 @@ static int menu_dialog_dokey (MUTTMENU *menu, int *ip)
   event_t ch;
   char *p;
 
-  ch = mutt_getch ();
+  do
+  {
+    ch = mutt_getch();
+  } while (ch.ch == -2);
 
   if (ch.ch < 0)
   {
@@ -1192,6 +1195,10 @@ int mutt_menuLoop (MUTTMENU *menu)
 	mutt_what_key ();
 	break;
 
+      case OP_CHECK_STATS:
+	mutt_check_stats ();
+	break;
+
       case OP_REDRAW:
 	clearok (stdscr, TRUE);
 	menu->redraw = REDRAW_FULL;
@@ -1202,6 +1209,11 @@ int mutt_menuLoop (MUTTMENU *menu)
 	menu->redraw = REDRAW_FULL;
 	break;
 
+      case OP_ERROR_HISTORY:
+	mutt_error_history_display ();
+	menu->redraw = REDRAW_FULL;
+	break;
+
       case OP_NULL:
 	km_error_key (menu->menu);
 	break;
