@@ -140,16 +140,17 @@ static int tls_socket_read (CONNECTION* conn, char* buf, size_t len)
     return -1;
   }
 
-  do {
+  do
+  {
     ret = gnutls_record_recv (data->state, buf, len);
-    if (ret < 0 && gnutls_error_is_fatal(ret) == 1)
-    {
-      mutt_error ("tls_socket_read (%s)", gnutls_strerror (ret));
-      mutt_sleep (4);
-      return -1;
-    }
+  } while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+
+  if (ret < 0)
+  {
+    mutt_error ("tls_socket_read (%s)", gnutls_strerror (ret));
+    mutt_sleep (4);
+    return -1;
   }
-  while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 
   return ret;
 }
@@ -169,17 +170,18 @@ static int tls_socket_write (CONNECTION* conn, const char* buf, size_t len)
 
   do
   {
-    ret = gnutls_record_send (data->state, buf + sent, len - sent);
+    do
+    {
+      ret = gnutls_record_send (data->state, buf + sent, len - sent);
+    } while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+
     if (ret < 0)
     {
-      if (gnutls_error_is_fatal(ret) == 1)
-      {
-	mutt_error ("tls_socket_write (%s)", gnutls_strerror (ret));
-	mutt_sleep (4);
-	return -1;
-      }
-      return ret;
+      mutt_error ("tls_socket_write (%s)", gnutls_strerror (ret));
+      mutt_sleep (4);
+      return -1;
     }
+
     sent += ret;
   } while (sent < len);
 
