@@ -164,7 +164,6 @@ static pid_t pgp_invoke (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			 short need_passphrase,
 			 const char *fname,
 			 const char *sig_fname,
-			 const char *signas,
 			 const char *ids,
 			 const char *format)
 {
@@ -179,7 +178,10 @@ static pid_t pgp_invoke (FILE **pgpin, FILE **pgpout, FILE **pgperr,
   cctx.need_passphrase = need_passphrase;
   cctx.fname	       = fname;
   cctx.sig_fname       = sig_fname;
-  cctx.signas	       = signas;
+  if (PgpSignAs && *PgpSignAs)
+    cctx.signas	       = PgpSignAs;
+  else
+    cctx.signas	       = PgpDefaultKey;
   cctx.ids	       = ids;
   
   mutt_pgp_command (cmd, sizeof (cmd), &cctx, format);
@@ -196,13 +198,12 @@ static pid_t pgp_invoke (FILE **pgpin, FILE **pgpout, FILE **pgperr,
  *
  */
 
-
 pid_t pgp_invoke_decode (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			 int pgpinfd, int pgpoutfd, int pgperrfd, 
 			 const char *fname, short need_passphrase)
 {
   return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     need_passphrase, fname, NULL, PgpSignAs, NULL, 
+		     need_passphrase, fname, NULL, NULL,
 		     PgpDecodeCommand);
 }
 
@@ -211,7 +212,7 @@ pid_t pgp_invoke_verify (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			 const char *fname, const char *sig_fname)
 {
   return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     0, fname, sig_fname, PgpSignAs, NULL, PgpVerifyCommand);
+		     0, fname, sig_fname, NULL, PgpVerifyCommand);
 }
 
 pid_t pgp_invoke_decrypt (FILE **pgpin, FILE **pgpout, FILE **pgperr,
@@ -219,7 +220,7 @@ pid_t pgp_invoke_decrypt (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			  const char *fname)
 {
   return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     1, fname, NULL, PgpSignAs, NULL, PgpDecryptCommand);
+		     1, fname, NULL, NULL, PgpDecryptCommand);
 }
 
 pid_t pgp_invoke_sign (FILE **pgpin, FILE **pgpout, FILE **pgperr,
@@ -227,7 +228,7 @@ pid_t pgp_invoke_sign (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 		       const char *fname)
 {
   return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     1, fname, NULL, PgpSignAs, NULL, PgpSignCommand);
+		     1, fname, NULL, NULL, PgpSignCommand);
 }
 
 
@@ -237,11 +238,11 @@ pid_t pgp_invoke_encrypt (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 {
   if (sign)
     return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		       1, fname, NULL, PgpSignAs, uids, 
+		       1, fname, NULL, uids,
 		       PgpEncryptSignCommand);
   else
     return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		       0, fname, NULL, PgpSignAs, uids,
+		       0, fname, NULL, uids,
 		       PgpEncryptOnlyCommand);
 }
 
@@ -251,11 +252,11 @@ pid_t pgp_invoke_traditional (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 {
   if (flags & ENCRYPT)
     return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		       flags & SIGN ? 1 : 0, fname, NULL, PgpSignAs, uids, 
+		       flags & SIGN ? 1 : 0, fname, NULL, uids,
 		       flags & SIGN ? PgpEncryptSignCommand : PgpEncryptOnlyCommand);
   else
     return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		       1, fname, NULL, PgpSignAs, NULL,
+		       1, fname, NULL, NULL,
 		       PgpClearSignCommand);
 }
 
@@ -270,7 +271,10 @@ void pgp_invoke_import (const char *fname)
   
   mutt_quote_filename (_fname, sizeof (_fname), fname);
   cctx.fname = _fname;
-  cctx.signas = PgpSignAs;
+  if (PgpSignAs && *PgpSignAs)
+    cctx.signas	       = PgpSignAs;
+  else
+    cctx.signas	       = PgpDefaultKey;
   
   mutt_pgp_command (cmd, sizeof (cmd), &cctx, PgpImportCommand);
   mutt_system (cmd);
@@ -321,7 +325,7 @@ pid_t pgp_invoke_export (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			 const char *uids)
 {
   return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     0, NULL, NULL, PgpSignAs, uids,
+		     0, NULL, NULL, uids,
 		     PgpExportCommand);
 }
 
@@ -330,7 +334,7 @@ pid_t pgp_invoke_verify_key (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			     const char *uids)
 {
   return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     0, NULL, NULL, PgpSignAs, uids,
+		     0, NULL, NULL, uids,
 		     PgpVerifyKeyCommand);
 }
 
@@ -338,21 +342,25 @@ pid_t pgp_invoke_list_keys (FILE **pgpin, FILE **pgpout, FILE **pgperr,
 			    int pgpinfd, int pgpoutfd, int pgperrfd, 
 			    pgp_ring_t keyring, LIST *hints)
 {
-  char uids[HUGE_STRING];
-  char tmpuids[HUGE_STRING];
+  BUFFER *uids;
   char quoted[HUGE_STRING];
-  
-  *uids = '\0';
-  
+  pid_t rc;
+
+  uids = mutt_buffer_pool_get ();
+
   for (; hints; hints = hints->next)
   {
     mutt_quote_filename (quoted, sizeof (quoted), (char *) hints->data);
-    snprintf (tmpuids, sizeof (tmpuids), "%s %s", uids, quoted);
-    strcpy (uids, tmpuids);	/* __STRCPY_CHECKED__ */
+    mutt_buffer_addstr (uids, quoted);
+    if (hints->next)
+      mutt_buffer_addch (uids, ' ');
   }
 
-  return pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
-		     0, NULL, NULL, PgpSignAs, uids,
+  rc = pgp_invoke (pgpin, pgpout, pgperr, pgpinfd, pgpoutfd, pgperrfd,
+                   0, NULL, NULL, mutt_b2s (uids),
 		     keyring == PGP_SECRING ? PgpListSecringCommand :
 		     PgpListPubringCommand);
+
+  mutt_buffer_pool_release (&uids);
+  return rc;
 }
