@@ -298,7 +298,7 @@ int mutt_get_postponed (CONTEXT *ctx, HEADER *hdr, HEADER **cur, char *fcc, size
     {
       if (ctx)
       {
-	/* if a mailbox is currently open, look to see if the orignal message
+	/* if a mailbox is currently open, look to see if the original message
 	   the user attempted to reply to is in this mailbox */
 	p = skip_email_wsp(tmp->data + 18);
 	if (!ctx->id_hash)
@@ -531,7 +531,7 @@ int mutt_parse_crypt_hdr (const char *p, int set_empty_signas, int crypt_app)
   if ((WithCrypto & APPLICATION_SMIME) && (crypt_app == APPLICATION_SMIME)
       && (flags & SIGN)
       && (set_empty_signas || *sign_as))
-    mutt_str_replace (&SmimeDefaultKey, sign_as);
+    mutt_str_replace (&SmimeSignAs, sign_as);
 
   return flags;
 }
