@@ -199,8 +199,7 @@ static const char * query_format_str (char *dest, size_t destlen, size_t col, in
   {
   case 'a':
     rfc822_write_address (buf2, sizeof (buf2), query->addr, 1);
-    snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-    snprintf (dest, destlen, tmp, buf2);
+    mutt_format_s (dest, destlen, fmt, buf2);
     break;
   case 'c':
     snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
@@ -208,16 +207,12 @@ static const char * query_format_str (char *dest, size_t destlen, size_t col, in
     break;
   case 'e':
     if (!optional)
-    {
-      snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-      snprintf (dest, destlen, tmp, NONULL (query->other));
-    }
+      mutt_format_s (dest, destlen, fmt, NONULL (query->other));
     else if (!query->other || !*query->other)
       optional = 0;
     break;
   case 'n':
-    snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-    snprintf (dest, destlen, tmp, NONULL (query->name));
+    mutt_format_s (dest, destlen, fmt, NONULL (query->name));
     break;
   case 't':
     snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
@@ -318,15 +313,6 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
   char helpstr[LONG_STRING];
   char title[STRING];
 
-  snprintf (title, sizeof (title), _("Query")); /* FIXME */
-
-  menu = mutt_new_menu (MENU_QUERY);
-  menu->make_entry = query_entry;
-  menu->search = query_search;
-  menu->tag = query_tag;
-  menu->title = title;
-  menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_QUERY, QueryHelp);
-
   if (results == NULL)
   {
     /* Prompt for Query */
@@ -340,6 +326,14 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
   {
     snprintf (title, sizeof (title), _("Query '%s'"), buf);
 
+    menu = mutt_new_menu (MENU_QUERY);
+    menu->make_entry = query_entry;
+    menu->search = query_search;
+    menu->tag = query_tag;
+    menu->title = title;
+    menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_QUERY, QueryHelp);
+    mutt_push_current_menu (menu);
+
     /* count the number of results */
     for (queryp = results; queryp; queryp = queryp->next)
       menu->max++;
@@ -382,6 +376,7 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 
 
 	      menu->current = 0;
+              mutt_pop_current_menu (menu);
 	      mutt_menuDestroy (&menu);
 	      menu = mutt_new_menu (MENU_QUERY);
 	      menu->make_entry = query_entry;
@@ -389,6 +384,7 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 	      menu->tag = query_tag;
 	      menu->title = title;
 	      menu->help = mutt_compile_help (helpstr, sizeof (helpstr), MENU_QUERY, QueryHelp);
+              mutt_push_current_menu (menu);
 
 	      /* count the number of results */
 	      for (queryp = results; queryp; queryp = queryp->next)
@@ -533,10 +529,8 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 
     free_query (&results);
     FREE (&QueryTable);
-    
-    /* tell whoever called me to redraw the screen when I return */
-    set_option (OPTNEEDREDRAW);
-  }
 
-  mutt_menuDestroy (&menu);
+    mutt_pop_current_menu (menu);
+    mutt_menuDestroy (&menu);
+  }
 }
