@@ -926,33 +926,51 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, HEADER *hdr,
 
 static void mutt_generate_recvattach_list (ATTACH_CONTEXT *actx,
                                            HEADER *hdr,
-                                           BODY *m,
+                                           BODY *parts,
                                            FILE *fp,
                                            int parent_type,
                                            int level,
                                            int decrypted)
 {
   ATTACHPTR *new;
+  BODY *m;
   BODY *new_body = NULL;
   FILE *new_fp = NULL;
-  int need_secured, secured;
+  int type, need_secured, secured;
 
-  for (; m; m = m->next)
+  for (m = parts; m; m = m->next)
   {
     need_secured = secured = 0;
 
     if ((WithCrypto & APPLICATION_SMIME) &&
-        mutt_is_application_smime (m))
+        (type = mutt_is_application_smime (m)))
     {
       need_secured = 1;
 
-      if (!crypt_valid_passphrase (APPLICATION_SMIME))
-        goto decrypt_failed;
+      if (type & ENCRYPT)
+      {
+        if (!crypt_valid_passphrase (APPLICATION_SMIME))
+          goto decrypt_failed;
 
-      if (hdr->env)
-        crypt_smime_getkeys (hdr->env);
+        if (hdr->env)
+          crypt_smime_getkeys (hdr->env);
+      }
 
       secured = !crypt_smime_decrypt_mime (fp, &new_fp, m, &new_body);
+      /* If the decrypt/verify-opaque doesn't generate mime output, an
+       * empty text/plain type will still be returned by
+       * mutt_read_mime_header().  We can't distinguish an actual part
+       * from a failure, so only use a text/plain that results from a single
+       * top-level part. */
+      if (secured &&
+          new_body->type == TYPETEXT &&
+          !ascii_strcasecmp ("plain", new_body->subtype) &&
+          (parts != m || m->next))
+      {
+        mutt_free_body (&new_body);
+        safe_fclose (&new_fp);
+        goto decrypt_failed;
+      }
 
       /* S/MIME nesting */
       if ((mutt_is_application_smime (new_body) & SMIMEOPAQUE) == SMIMEOPAQUE)
@@ -970,7 +988,7 @@ static void mutt_generate_recvattach_list (ATTACH_CONTEXT *actx,
         safe_fclose (&outer_fp);
       }
 
-      if (secured)
+      if (secured && (type & ENCRYPT))
         hdr->security |= SMIMEENCRYPT;
     }
 
