@@ -66,8 +66,9 @@ enum {
   CAPMAX
 };
 
-#ifdef USE_SASL
 static int smtp_auth (CONNECTION* conn);
+static int smtp_auth_oauth (CONNECTION* conn);
+#ifdef USE_SASL
 static int smtp_auth_sasl (CONNECTION* conn, const char* mechanisms);
 #endif
 
@@ -495,19 +496,12 @@ static int smtp_open (CONNECTION* conn)
       return -1;
     }
 
-#ifdef USE_SASL
     return smtp_auth (conn);
-#else
-    mutt_error (_("SMTP authentication requires SASL"));
-    mutt_sleep (1);
-    return -1;
-#endif /* USE_SASL */
   }
 
   return 0;
 }
 
-#ifdef USE_SASL
 static int smtp_auth (CONNECTION* conn)
 {
   int r = SMTP_AUTH_UNAVAIL;
@@ -528,21 +522,41 @@ static int smtp_auth (CONNECTION* conn)
 
       dprint (2, (debugfile, "smtp_authenticate: Trying method %s\n", method));
 
-      r = smtp_auth_sasl (conn, method);
-      
+      if (!strcmp (method, "oauthbearer"))
+      {
+	r = smtp_auth_oauth (conn);
+      }
+      else 
+      {
+#ifdef USE_SASL
+	r = smtp_auth_sasl (conn, method);
+#else
+	mutt_error (_("SMTP authentication method %s requires SASL"), method);
+	mutt_sleep (1);
+	continue;
+#endif
+      }
       if (r == SMTP_AUTH_FAIL && delim)
       {
-        mutt_error (_("%s authentication failed, trying next method"), method);
-        mutt_sleep (1);
+	mutt_error (_("%s authentication failed, trying next method"), method);
+	mutt_sleep (1);
       }
       else if (r != SMTP_AUTH_UNAVAIL)
-        break;
+	break;
     }
 
     FREE (&methods);
   }
   else
+  {
+#ifdef USE_SASL
     r = smtp_auth_sasl (conn, AuthMechs);
+#else
+    mutt_error (_("SMTP authentication requires SASL"));
+    mutt_sleep (1);
+    r = SMTP_AUTH_UNAVAIL;
+#endif
+  }
 
   if (r != SMTP_AUTH_SUCCESS)
     mutt_account_unsetpass (&conn->account);
@@ -561,6 +575,7 @@ static int smtp_auth (CONNECTION* conn)
   return r == SMTP_AUTH_SUCCESS ? 0 : -1;
 }
 
+#ifdef USE_SASL
 static int smtp_auth_sasl (CONNECTION* conn, const char* mechlist)
 {
   sasl_conn_t* saslconn;
@@ -663,3 +678,36 @@ fail:
   return SMTP_AUTH_FAIL;
 }
 #endif /* USE_SASL */
+
+
+/* smtp_auth_oauth: AUTH=OAUTHBEARER support. See RFC 7628 */
+static int smtp_auth_oauth (CONNECTION* conn)
+{
+  char* ibuf = NULL;
+  char* oauthbearer = NULL;
+  int ilen;
+  int rc;
+
+  mutt_message _("Authenticating (OAUTHBEARER)...");
+
+  /* We get the access token from the smtp_oauth_refresh_command */
+  oauthbearer = mutt_account_getoauthbearer (&conn->account);
+  if (oauthbearer == NULL)
+    return SMTP_AUTH_FAIL;
+
+  ilen = strlen (oauthbearer) + 30;
+  ibuf = safe_malloc (ilen);
+
+  snprintf (ibuf, ilen, "AUTH OAUTHBEARER %s\r\n", oauthbearer);
+
+  rc = mutt_socket_write (conn, ibuf);
+  FREE (&oauthbearer);
+  FREE (&ibuf);
+
+  if (rc == -1)
+    return SMTP_AUTH_FAIL;
+  if (smtp_get_resp (conn) != 0) 
+    return SMTP_AUTH_FAIL;
+
+  return SMTP_AUTH_SUCCESS;
+}
